*! version 0.1  26jul2013  Michael Stepner, michaelstepner@gmail.com

program define vam_policysim

	version 11

	set more off

	syntax using/ , NTeachers(integer) NYears(integer) NClasses(numlist integer >=1) NStudents(real) decisionyears(numlist integer >=1 ascending) cut(real) impacthorizon(integer) [ max_futuresumhorizon(integer -1) survival(numlist min=0) output(string) append driftlimit(integer -1) ONLYGROUPs(numlist integer >=1) zero_var_ind zero_var_class ]

	* Error checks
	if (`"`output'"'=="" & "`append'"!="") {
		di as error "Cannot specify append without output()"
		exit 198
	}
	
	capture confirm new file `"`output'"'
	if _rc==603 {
		di as error `"The specified output file is not a valid file path"'
		exit 601
	}
	
	if (`max_futuresumhorizon'<0) local max_futuresumhorizon=`nyears'
	
	* Fill in survival vector with 1s if not specified
	if ("`survival'"=="") {
		forvalues i=1/`nyears' {
			local survival 1 `survival'
		}
	}
	
	* Load csv
	clear
	qui insheet using `using', comma
	
	* Define group labels
	unab rawgroups : cov_lag*
	local groups=subinstr("`rawgroups'","cov_lag_","",.)
	
	if ("`groups'"=="cov_lag") {
		label define group 1 "n/a"
	}
	else {
		tokenize "`groups'"
		local groupnum=1
		while "`1'"!="" {
			label define group `groupnum' `1', add
			local ++groupnum
			mac shift
		}
	}
	
	* Define decision rule label
	label define decisionrule 1 "predicted next year" 2 "predicted future" 3 "unweighted avg observed"

	* Compute policy impacts
	if ("`onlygroups'"=="") mata:onlygroups=.
	else mata:onlygroups=strtoreal(tokens(st_local("onlygroups")))
	
	set seed 15977701
	mata:policysim(strtoreal(st_local("nteachers")),strtoreal(st_local("nyears")),strtoreal(tokens(st_local("nclasses"))),strtoreal(st_local("nstudents")),strtoreal(tokens(st_local("survival"))),strtoreal(tokens(st_local("decisionyears"))),strtoreal(st_local("cut")),strtoreal(st_local("impacthorizon")),strtoreal(st_local("max_futuresumhorizon")),strtoreal(st_local("driftlimit")),onlygroups,strlen(st_local("zero_var_ind")),strlen(st_local("zero_var_class")))

	* Apply labels
	label values group group
	label values decisionrule decisionrule

	* Save to output file
	if (`"`output'"')!="" {
		if ("`append'"!="") append using `"`output'"', nolabel
		save `"`output'"', replace
	}
end

version 11
set matastrict on

/************************************************************/
/***** Mata programs that are specific to vam_policysim *****/
/************************************************************/

mata:
void policysim(real scalar nteachers, real scalar nyears, real rowvector nclasses, real scalar nstudents, real rowvector survival, real rowvector decisionyears, real scalar cut, real scalar impacthorizon, real scalar max_futuresumhorizon, real scalar driftlimit, real rowvector onlygroups, real scalar zero_var_ind, real scalar zero_var_class) {

	// Perform error checks
	if (length(survival)!=nyears) _error("The number of elements in the cumulative survival vector must equal nyears().")
	if (max(decisionyears)+impacthorizon>nyears) _error("nyears must be greater than max(decisionyears) + impacthorizon.")
	if (cut<0 | cut>1) _error("cut() must be between 0 and 1.")
	
	real scalar i
	for (i=1; i<nyears; i++) {
		if (survival[i]<survival[i+1]) _error("The cumulative survival vector must be weakly descending.")
	}

	// Load variance-covariance data from stata dataset
	real scalar n_obs
	n_obs=st_nobs()

	real matrix cov_lag
	real rowvector var_total
	real rowvector var_ind
	real rowvector cov_sameyear
	real rowvector var_class
	
	cov_lag=st_data((1,n_obs-1),"cov_lag*")[.,onlygroups]
	
	var_total=st_data(n_obs,"var_total*")[.,onlygroups]
	
	if (zero_var_ind==0) var_ind=st_data(n_obs,"var_ind*")[.,onlygroups]
	else var_ind=J(1,length(onlygroups),0)
	
	cov_sameyear=st_data(n_obs,"cov_sameyear*")[.,onlygroups]
	
	if (zero_var_class==0) var_class=var_total-var_ind-cov_sameyear
	else var_class=J(1,length(onlygroups),0)
	
	// Create group identifier vector
	real rowvector groups
	if (onlygroups=.) groups=(1..cols(var_total))
	else groups=onlygroups
	
	// Prepare stata dataset for storing policy impacts
	stata("drop _all")
	real rowvector null
	null=st_addvar("int",("group","nclasses","decisionyear","decisionrule"))
	null=st_addvar("float",("effect","exacteffect_nextyear"))
	null=st_addvar("float","year":+strofreal(1..nyears):+"meanVA")
	
	// For each "# of classes taught per year":
	real scalar c
	real scalar classes
	for (c=1; c<=length(nclasses); c++) {
		classes=nclasses[c]
		
		//Compute variance, which will appear on diagonal of VCV matrix
		real rowvector variance
		variance=cov_sameyear + var_class / classes + var_ind / (classes*nstudents)
		
		// Loop over each group
		real rowvector vcv_vector
		real matrix vcv
		real matrix simscores
		real scalar j
		
		real scalar decisionyear
		real scalar decisionyear_p1
		real rowvector predecision
		real rowvector postdecision
		real rowvector postdecision_impacthorizon
		
		real colvector discounts
		real colvector discounts_impacthorizon
				
		real rowvector weights
		real scalar exacteffect_nextyear
		
		real scalar futuresumhorizon	
		real matrix predtv
		real scalar t
		
		real colvector predtv_nextyear
		real colvector predtv_dsum
		real colvector predtv_avgobserved
		
		real rowvector effecteachyear_nextyear
		real rowvector effecteachyear_discfuture
		real rowvector effecteachyear_avgobserved
		
		real scalar effect_nextyear
		real scalar effect_discfuture
		real scalar effect_avgobserved
		
		real rowvector id
		
		for (i=1; i<=length(groups); i++) {
		
			// Define the vcv_vector (vector, because it will be stripe-diagonalized)
			if (driftlimit>0) vcv_vector=create_m(cov_lag[.,i],variance[i],n_obs-1,driftlimit)
			else vcv_vector=create_m(cov_lag[.,i],variance[i])
			
			// Adjust length of vcv_vector. 
			//     note: length(vcv_vector)==n_obs. new length=nyears
			if (n_obs>nyears) vcv_vector=vcv_vector[1..nyears]
			else if (n_obs<nyears) vcv_vector=vcv_vector,J(1,nyears-n_obs,vcv_vector[n_obs])
			
			// Create VCV matrix
			vcv=vectorToStripeDiag(vcv_vector)
	
			// Generate Monte Carlo-simulated scores using multivariate normal draws from N(0,VCV)
			simscores=drawnorm(nteachers,vcv)
			
			// For each decision year:
			for(j=1; j<=length(decisionyears); j++) {
			
				// Define useful decision year periods
				decisionyear=decisionyears[j]
				decisionyear_p1=decisionyear+1
				predecision=(1::decisionyear)
				postdecision=(decisionyear_p1..nyears)
				postdecision_impacthorizon=(decisionyear_p1..decisionyear+impacthorizon)
				
				// compute vector of discounts, for discounting back to the decisionyear
				discounts=survival[postdecision]':/survival[decisionyear]
				discounts_impacthorizon=discounts[1..impacthorizon]
				
				// Compute the exact (non Monte Carlo) effect on next year VA
				weights=computeweights(vcv, decisionyear_p1, predecision)
				exacteffect_nextyear=(-1 * normalden(invnormal(cut)) / normal(invnormal(cut))) * sqrt( weights*vcv[predecision,predecision]*weights' )
				
				// Compute predicted tv using only the years prior to or equal to the decisionyear
				futuresumhorizon=min((nyears,decisionyear+max_futuresumhorizon))-decisionyear
				predtv=J(nteachers,futuresumhorizon,.)
				
				for (t=decisionyear_p1; t<=decisionyear+futuresumhorizon; t++) {
					weights=computeweights(vcv, t, predecision)
					
					predtv[.,t-decisionyear]=simscores[.,predecision]*weights'
				}
				
				// store predicted tv for next year
				predtv_nextyear=predtv[.,1]
				
				// store (discounted) sum of predicted future tv
				predtv_dsum=predtv * discounts[1..futuresumhorizon]
				
				// store predicted tv based on unweighted average of observed tv (which is equivalent to selecting on raw sum of observed tv)
				predtv_avgobserved=rowsum(simscores[.,predecision])
				
				// compute the effect of deselecting the bottom teachers				
				effecteachyear_nextyear=	mean( select(simscores, predtv_nextyear:<=mm_quantile(predtv_nextyear,1,cut)		))
				effecteachyear_discfuture=	mean( select(simscores, predtv_dsum:<=mm_quantile(predtv_dsum,1,cut)				))
				effecteachyear_avgobserved=	mean( select(simscores, predtv_avgobserved:<=mm_quantile(predtv_avgobserved,1,cut)	))
				
				effect_nextyear=	effecteachyear_nextyear[.,postdecision_impacthorizon] * discounts_impacthorizon
				effect_discfuture=	effecteachyear_discfuture[.,postdecision_impacthorizon] * discounts_impacthorizon
				effect_avgobserved= effecteachyear_avgobserved[.,postdecision_impacthorizon] * discounts_impacthorizon

				// store effects
				st_addobs(3)
				id=(groups[i],classes,decisionyear)
				st_store(st_nobs()-2::st_nobs(),(1..6+nyears),((id,1,effect_nextyear,exacteffect_nextyear,effecteachyear_nextyear)\(id,2,effect_discfuture,.,effecteachyear_discfuture)\(id,3,effect_avgobserved,.,effecteachyear_avgobserved)))
			}	
		}
	}
}

real matrix drawnorm(real scalar n, real matrix V)
{
	return(invnormal(uniform(n,cols(V)))*cholesky(V)')
}

end

/***********************************************************/
/***** Mata programs copied from vam.ado, version 2.0 *****/
/***********************************************************/

mata:
real rowvector computeweights(real matrix M, real scalar i, real colvector c, | real colvector weights) {
	
	// construct matrix A which is used to select the relevant elements of M in constructing the VCV matrix
	real matrix temp
	real matrix A
	temp=designmatrix(c)
	A = temp, J(rows(c),cols(M)-cols(temp),0)
	
	// use A to select elements of M and build the VCV.  The second term adjusts the diagonal elements of the VCV matrix to account for the class-level and individual-level shocks
	real matrix vcv
	if (args()==4) vcv=A*M*A' + diag(1:/weights)
	else vcv=A*M*A'
	
	// phi is the vector of autocovariances, selected correctly using the matrix A.
	real rowvector phi
	phi=M[i,.]*A'
	
	// return the vector of weights
	return (phi*invsym(vcv))
	
}

real matrix compute_cov_corr(string scalar scores_var, string scalar weight_var, real scalar dim, string scalar teacher_var) {
	
	// pre-allocate matrix
	real matrix CC
	CC = J(dim,4,.)
	
	// Fill cov's and corr's: between time t and t+i
	real scalar i
	real scalar tstat
	for (i=1; i<=dim; i++) {
		// check that there are >=2 obs, in order to compute covariance
		stata(invtokens(("quietly count if !missing(",scores_var,",f",strofreal(i),".",scores_var,")"),""))
		if (st_numscalar("r(N)")>1) {
			stata(invtokens(("quietly corr ",scores_var," f",strofreal(i),".",scores_var," [aw=",weight_var,"+f",strofreal(i),".",weight_var,"], cov"),""))
			CC[i,1]=st_numscalar("r(cov_12)")
			CC[i,2]=CC[i,1] / ( sqrt(st_numscalar("r(Var_1)")) * sqrt(st_numscalar("r(Var_2)")) )
		}
		CC[i,3]=st_numscalar("r(N)")
		
		// Compute SE for covariance estimate
		if (st_numscalar("r(N)")>1) {
			stata(invtokens(("quietly reg ",scores_var," f",strofreal(i),".",scores_var," [aw=",weight_var,"+f",strofreal(i),".",weight_var,"], cluster(",teacher_var,")"),""))
			tstat=st_matrix("e(b)")[1,1] / sqrt( st_matrix("e(V)")[1,1] )
			CC[i,4]=abs(CC[i,1]/tstat)
		}
	}

	return (CC)
}

real rowvector create_m(real colvector lag_covariances, real scalar cov_sameyear, | real scalar lagdim, real scalar driftlimit) {

	real rowvector m

	if (args()==2)	m=cov_sameyear,lag_covariances'
	else {
		if (length(lag_covariances)<driftlimit) _error("driftlimit specified is higher than the number of lags in the dataset")
		m=cov_sameyear,lag_covariances'[1..driftlimit],J(1,lagdim-driftlimit,lag_covariances[driftlimit])
	}
	
	return (m)
}

void check_m_nomissing(real rowvector m) {
	if (missing(m)>0) _error("covariance vector contains missing values")
}

real matrix vectorToStripeDiag(real vector m) {
	real scalar dim
	dim = length(m)
	
	// pre-allocate matrix M
	real matrix M
	M=J(dim,dim,.)
	
	// fill lower triangle of M
	real scalar i
	real scalar j
	for (i=1; i<=dim; i++) {
		for (j=i; j<=dim; j++) {
			M[j,i]=m[j-i+1]
		}
	}
	
	_makesymmetric(M)
	return (M)
}

real matrix rightAppendMatrices(real matrix A, real matrix B) {
	real scalar rA
	real scalar rB
	rA=rows(A)
	rB=rows(B)
	
	if (rA==rB)		return (A,B)
	else if (rA<rB)	return ( ( A \ J(rB-rA,cols(A),.) ) , B )
	else			return ( A , ( B \ J(rA-rB,cols(B),.) ) )
}

void saveVariancesToDataset(real matrix cov_lag_accum, real matrix corr_lag_accum, real matrix obs_lag_accum, real matrix cov_se_lag_accum, real rowvector var_total_accum, real rowvector var_class_accum, real rowvector var_ind_accum, real rowvector cov_sameyear_accum, real rowvector corr_sameyear_accum, real rowvector obs_sameyear_accum, string rowvector suffixes) {

	stata("clear")
	
	// count number of lags, create correct number of obs, generate variable for number of lags
	real scalar n_lags
	n_lags=rows(cov_lag_accum)
	
	real scalar null
	null=st_addvar("int","lag")
	
	st_addobs(n_lags)
	stata("qui replace lag=_n")
	st_addobs(1)
	
	// generate output variables
	st_store(1::n_lags, st_addvar("float", "cov_lag":+suffixes), cov_lag_accum)
	st_store(1::n_lags, st_addvar("float", "corr_lag":+suffixes), corr_lag_accum)
	st_store(1::n_lags, st_addvar("float", "obs_lag":+suffixes), obs_lag_accum)
	st_store(1::n_lags, st_addvar("float", "cov_se_lag":+suffixes), cov_se_lag_accum)
	st_store(n_lags+1, st_addvar("float", "var_total":+suffixes), var_total_accum)
	st_store(n_lags+1, st_addvar("float", "var_class":+suffixes), var_class_accum)
	st_store(n_lags+1, st_addvar("float", "var_ind":+suffixes), var_ind_accum)
	st_store(n_lags+1, st_addvar("float", "cov_sameyear":+suffixes), cov_sameyear_accum)
	st_store(n_lags+1, st_addvar("float", "corr_sameyear":+suffixes), corr_sameyear_accum)
	st_store(n_lags+1, st_addvar("float", "obs_sameyear":+suffixes), obs_sameyear_accum)
}

real scalar driftcalc(real matrix M, real scalar i, real colvector c, real colvector weights, real colvector scores) {

	// b is the vector of weights
	real rowvector b
	b=computeweights(M, i, c, weights)
	
	// return the computed tv estimate
	return (b*scores)
	
}

void driftcalclist(real matrix M, string scalar teacher_var, string scalar time_var, string scalar scores_var, string scalar weights_var, string scalar teacherobs_var, string scalar va_var, | string scalar va_2yr_l_var, string scalar va_2yr_f_var, string scalar va_ss_var) {

	real scalar quasi
	if (args()==7) quasi=0
	else if (args()==10) quasi=1
	else _error("The mata command driftcalclist must either be called with no quasi-experiment variables or the full set of 3 quasi-experiment variables.)")

	real scalar nobs
	nobs=st_nobs()
	
	// get variable indices for the variables referenced in the loop (referring by index speeds up the loop)
	real scalar teacher_var_ind
	real scalar time_var_ind
	real scalar teacherobs_var_ind
	real scalar va_var_ind
	teacher_var_ind=st_varindex(teacher_var)
	time_var_ind=st_varindex(time_var)
	teacherobs_var_ind=st_varindex(teacherobs_var)
	va_var_ind=st_varindex(va_var)
	if (quasi==1) {
		real scalar va_2yr_l_var_ind
		real scalar va_2yr_f_var_ind
		real scalar va_ss_var_ind
		va_2yr_l_var_ind=st_varindex(va_2yr_l_var)
		va_2yr_f_var_ind=st_varindex(va_2yr_f_var)
		va_ss_var_ind=st_varindex(va_ss_var)
	}
	
	// create views of the variables we need
	real matrix Z
	st_view(Z=.,.,(teacher_var,time_var,weights_var,scores_var))
	
	
	// Declarations
	real scalar obs
	real scalar teacher
	real scalar obs_teacher
	real scalar time
	real scalar new_teacher
	real scalar new_time
	real scalar year_index
	real matrix Z_teacher
	real matrix Z_obs
	real matrix Z_quasi
	
	// set missing b/c referenced in first loop's if statement
	teacher=.
	time=.
	
	// Loop over all observations
	for (obs=1; obs<=nobs; obs++) {
	
		new_teacher=_st_data(obs,teacher_var_ind)
		new_time=_st_data(obs,time_var_ind)
	
		// Only perform calculations if we've reached a new teacher-year
		if (new_time != time | new_teacher != teacher) {
	
			// save new time id
			time=new_time
		
			// If we've reached a new teacher
			if (new_teacher != teacher) {
			
				// save new teacher id
				teacher=new_teacher
				
				// save number of observations for that teacher
				obs_teacher=_st_data(obs,teacherobs_var_ind)
				
				// select subview of Z, Z_teacher, which only contains the correct teacher's data
				st_subview(Z_teacher=., Z, (obs,obs+obs_teacher-1), .)
				
				// define teacher-specific scalar which indexes first year of teaching at 1
				year_index=min(Z_teacher[.,2])-1
				
			}
		
			// remove the rows of Z_teacher corresponding to current year
			Z_obs=select(Z_teacher, Z_teacher[.,2]:!=time)
					
			// remove rows of Z_obs that do not have score data
			Z_obs=select(Z_obs, Z_obs[.,4]:!=.)
			
			// if there are actually observations in other years, compute VA
			if (rows(Z_obs) > 0) {	
				st_store(obs,va_var_ind,driftcalc(M,time-year_index,Z_obs[.,2]:-year_index,Z_obs[.,3],Z_obs[.,4]))
			}
			
			if (quasi==1) {
				// remove the rows of Z_obs corresponding to the previous year
				Z_quasi=select(Z_obs, Z_obs[.,2]:!=time-1)
				if (rows(Z_quasi) > 0) {
					st_store(obs,va_2yr_l_var_ind,driftcalc(M,time-year_index,Z_quasi[.,2]:-year_index,Z_quasi[.,3],Z_quasi[.,4]))
				}
				
				// remove the rows of Z_obs corresponding to the next year
				Z_quasi=select(Z_obs, Z_obs[.,2]:!=time+1)
				if (rows(Z_quasi) > 0) {
					st_store(obs,va_2yr_f_var_ind,driftcalc(M,time-year_index,Z_quasi[.,2]:-year_index,Z_quasi[.,3],Z_quasi[.,4]))
				}
				
				// remove the rows of Z_obs corresponding to {t-2,t-1,t+1,t+2}
				Z_quasi=select(Z_obs, (Z_obs[.,2]:>time+2)+(Z_obs[.,2]:<time-2))
				if (rows(Z_quasi) > 0) {
					st_store(obs,va_ss_var_ind,driftcalc(M,time-year_index,Z_quasi[.,2]:-year_index,Z_quasi[.,3],Z_quasi[.,4]))
				}
			}		
		}
	}
}
end
